package fr.itsonus.bousoleplussbackend.controllers;

import fr.itsonus.bousoleplussbackend.models.Quiz;
import fr.itsonus.bousoleplussbackend.models.QuizScore;
import fr.itsonus.bousoleplussbackend.models.Response;
import fr.itsonus.bousoleplussbackend.payload.request.QuizRequest;
import fr.itsonus.bousoleplussbackend.repositories.QuestionRepository;
import fr.itsonus.bousoleplussbackend.repositories.QuizRepository;
import fr.itsonus.bousoleplussbackend.repositories.QuizScoreRepository;
import fr.itsonus.bousoleplussbackend.repositories.ResponseRepository;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.NoSuchElementException;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/quizzes")
@AllArgsConstructor
public class QuizController {

    private QuizRepository quizRepository;
    private ResponseRepository responseRepository;
    private QuestionRepository questionRepository;
    private QuizScoreRepository quizScoreRepository;

    @PostMapping("batch") // TODO add to REST representation
    public Quiz create(@Valid @RequestBody QuizRequest request) {
        var quiz = quizRepository.save(new Quiz());
        var responsesCreated = request.getResponses().stream().map(response -> {
            // TODO add correct exception with correct status code
            var question = questionRepository.findById(response.getQuestionId())
                    .orElseThrow(() -> new NoSuchElementException("No such question with id " + response.getQuestionId()));
            log.info("Saving {}", response);
            return responseRepository.save(
                    new Response()
                            .setScore(response.getScore())
                            .setComment(response.getComment())
                            .setQuiz(quiz)
                            .setQuestion(question));
        }).collect(Collectors.toSet());
        quiz.setResponses(responsesCreated);
        return quiz;
    }

    @GetMapping("{id}/scores")
    public Iterable<QuizScore> findScores(@PathVariable("id") Long id) {
        return this.quizScoreRepository.findAllByQuizId(id);
    }
}
